/*****************************************************************************
*                            ASTRO PROCEDURES                                *
*                                                                            *
*                19 September 1995,   Marc de Lignie                         *
******************************************************************************
ASTRO contains some frequently used procedures in astronomic software.
For using this library, include it in your make or project file. Further,
add the file astro.h to the include section of your C program.
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "astro.h"

#define IGREG (15+31L*(10+12L*1582))

/*
  r_rade converts x,y,z to right ascension and declination
*/
void r_rade(float *r, float *ra, float *de)
{
  float help;

  if (fabs(r[2])==1.) {
    *de= (r[2]==1.) ? PI/2. : -PI/2.;
    *ra=0.;           /* willekeurige waarde */
  }
  else {
    *de=atan(r[2]/sqrt(1-r[2]*r[2]));
    if (r[0]==0.) {
      help=r[1]/cos(*de);
      *ra=atan(help/sqrt(1.-help*help));
      if (r[1]<0.) *ra+=2.*PI;
    }
    else {
      *ra=atan(r[1]/r[0]);
      if (r[0]<0) *ra+=PI;
      else if (r[1]<0) *ra+=2.*PI;
    }
  }
}


/*
  rade_r converts right ascension and declination to x,y,z.
*/
void rade_r(float ra, float de, float *r)
{
  r[0]=cos(ra)*cos(de);
  r[1]=sin(ra)*cos(de);
  r[2]=sin(de);
}


/*
   Source: Numerical Recipes in C, Cambridge University Press, 1988
*/
long julianday(int mm, int id, int iyyy)
{
    long jul;
    int ja,jy,jm;

    if (iyyy == 0) {
      printf("JULDAY: there is no year zero.");
      exit(1);
    }
    if (iyyy < 0) ++iyyy;
    if (mm > 2) {
         jy=iyyy;
         jm=mm+1;
    } else {
         jy=iyyy-1;
         jm=mm+13;
    }
    jul = (long) (floor(365.25*jy)+floor(30.6001*jm)+id+1720995L);
    if (id+31L*(mm+12L*iyyy) >= IGREG) {
         ja=0.01*jy;
         jul += 2-ja+(int) (0.25*ja);
    }
    return jul;
}


/*
   Calculates solar longitude for equinox 2000.0.
   Source: Casper ter Kuile.
   Original source: Jean Meeus.
*/
float sol2000(long jul, float tut)
{
  double sl,sa,te,c1,c2,c3,c,stl,omega,sal;

  te=(jul-2415020L)/36525.;                      /* decimale eeuw vanaf 1900 */
  sl = 279.69668 + 36000.76892*te + .0003025*te*te;
  sa = 358.47583 + 35999.04975*te - .00015  *te*te - .0000033*te*te*te;
  sl = sl - floor(sl/360.)*360.;
  sa = sa - floor(sa/360.)*360.;
  sa = sa*PS;
  c1 = (1.91946 - .004789 * te - .000014*te*te) * sin(sa);
  c2 = (.020094 - .0001*te) * sin(2.*sa);
  c3 = .000293 * sin(3.*sa);
  c = c1 + c2 + c3;
/*
  Berekening  werkelijke  zonne-lengte  en  werkelijke  anomalie
*/
  stl = sl + c;
/*
  Berekening  schijnbare  zonne-lengte
*/
  omega = 259.18 - 1934.142*te;
  omega = omega - floor(omega/360.)*360.;
  sal = stl - .00569 - .00479 * sin(omega*PS);  /* zonslengte voor 12h UT */
  sal=sal+(tut-12.)/24.*360./365.25;
  sal=sal - .01396 * (100.*te - 100.);          /* precessie correctie */
  return sal;
}


/*
  Calculates the radiant from two meteor trails and the angle of
  convergence between the two trails. QQQ returns -1 when the the
  two trails do not intersect in backwards direction. QQQ returns -2
  when the distance of one of the trails to the radiant is larger than
  120 degrees (but all calculations are made). Otherwise QQQ returns 0.
*/
int qqq(struct meteor met1, struct meteor met2, float *radr, float *q)
{
  int retval=0;
  float v1[3], v2[3], len, l1[3], l2[3], cosq, inb1, inb2;
/*
  v1 en v2 zijn de normalen van de vlakken door de radiant, de oorsprong
  en de eerste respectievelijk tweede meteoor. radr is de radiantvector.
*/
  out_p(met1.rb, met1.re, v1);
  out_p(met2.rb, met2.re, v2);
  out_p(v1,v2,radr);
  len=sqrt(in_p(radr,radr));
  if (len==0) {
    *q=0;
    return 0;
  }
  radr[0]/=len;
  radr[1]/=len;                           /* Normalisatie op eenheidscirkel */
  radr[2]/=len;
  if (in_p(met1.rb,radr) < in_p(met1.re,radr)) {
    radr[0]=-radr[0];
    radr[1]=-radr[1];          /* Beginpunt moet dichtst bij radiant liggen */
    radr[2]=-radr[2];
  }
          /* Controleer of meteoren werkelijk achterwaarts verlengd snijden */
	  /* en of de afstand tot radiant niet groter is dan 120 graden     */
  inb1=in_p(met1.rb,radr);
  inb2=in_p(met2.rb,radr);
  if (inb2 < in_p(met2.re,radr)) retval=-1;
  if (inb1 < -0.5 || inb2 <-0.5) retval=-2;
/*
  l1 en l2 zijn de richtingsvectoren van de snijlijnen tussen het raakvlak aan
  de eenheidscirkel ter plekke van de radiant enerzijds en de vlakken v1 en v2
  anderzijds. De hoek tussen l1 en l2 is de convergentiehoek q.
*/
  out_p(v1,radr,l1);
  out_p(v2,radr,l2);
  cosq=in_p(l1,l2)/sqrt(in_p(l1,l1)*in_p(l2,l2));
  *q=acos(cosq);
  if (*q>PI/2) *q=PI-*q;                            /* Zoek de scherpe hoek */
  return retval;
}


/*
  Calculates the vector in-product
*/
float in_p(float *vec1, float *vec2)
{
  return vec1[0]*vec2[0]+vec1[1]*vec2[1]+vec1[2]*vec2[2];
}


/*
  Calculates the vector out-product
*/
void out_p(float *vec1, float *vec2, float *vecr)
{
  vecr[0]=vec1[1]*vec2[2]-vec1[2]*vec2[1];
  vecr[1]=vec1[2]*vec2[0]-vec1[0]*vec2[2];
  vecr[2]=vec1[0]*vec2[1]-vec1[1]*vec2[0];
}


/*
  Converts hhmmss.x or ggmmss.x to hh.xxxxx or gg.xxxxx, respectively
*/
float ghdec(float x)
{
  float gh,m,s,sign;

  if (x<0) {
    x*=-1.;
    sign=-1.;
  } else sign=+1.;
  x+=0.05;                                  /* rounding */
  gh=floor(x/10000.);
  m=floor((x-10000.*gh)/100.);
  s=x-10000.*gh-100.*m;
  return sign*(gh+(m*60.+s-0.05)/3600.);
}


/*
  Converts gg.xxxxx or hh.xxxxx to ggmmss.x or hhmmss.x respectively
*/
float decgh(float x)
{
  float gh,m,s,f,sign;

  if (x<0) {
    x*=-1.;
    sign=-1.;
  } else sign=+1.;
  gh=floor(x);
  f=60.*(x-gh);
  m=floor(f);
  s=60.*(f-m);
  return sign*(gh*10000.+m*100.+s);
}


void azhude(float az,float elv,float phi,float *u,float *de)
{
   float sinu;

   *de=sin(elv)*sin(phi)-cos(elv)*cos(az)*cos(phi);
   *de=asin(*de);
   *u=(sin(elv)*cos(phi)+cos(elv)*cos(az)*sin(phi))/cos(*de);
   *u=acos(*u);
   sinu=cos(elv)*sin(az)/cos(*de);
   if (sinu<0.) *u=2.*PI-*u;
   return;
}
